@@ -43,7 +43,7 @@ namespace embree
       last_resident_memory(0)
   {
     if (instance)
-      throw std::runtime_error("internal error: applicaton already created");
+      throw std::runtime_error("internal error: application already created");
 
     instance = this;
 
@@ -106,55 +106,6 @@ namespace embree
     assert(instance == this);
     instance = nullptr;
   }
-  
-  void Application::registerOptionAlias(const std::string& name, const std::string& alternativeName) {
-    commandLineOptionMap[alternativeName] = commandLineOptionMap[name];
-  }
-  
-  void Application::parseCommandLine(int argc, char** argv)
-  {
-    /* create stream for parsing */
-    Ref<ParseStream> stream = new ParseStream(new CommandLineStream(argc, argv));
-    
-    /* parse command line */  
-    parseCommandLine(stream, FileName());
-  }
-  
-  void Application::parseCommandLine(Ref<ParseStream> cin, const FileName& path)
-  {
-    while (true)
-    {
-      std::string tag = cin->getString();
-      if (tag == "") return;
-      std::string tag0 = tag;
-      
-      /* remove - or -- and lookup command line option */
-      if (tag.find("-") == 0) 
-      {
-        tag = tag.substr(1);
-        if (tag.find("-") == 0) tag = tag.substr(1);
-        auto option = commandLineOptionMap.find(tag);
-      
-        /* process command line option */
-        if (option != commandLineOptionMap.end()) {
-          option->second->parse(cin,path);
-          continue;
-        }
-      }
-      
-      /* handle unknown command line options */
-      std::cerr << "unknown command line parameter: " << tag0 << " ";
-      while (cin->peek() != "" && cin->peek()[0] != '-') std::cerr << cin->getString() << " ";
-      std::cerr << std::endl;
-    }
-  }
-
-  void Application::printCommandLineHelp()
-  {
-    for (auto& c : commandLineOptionList) {
-      std::cout << c->description << std::endl;
-    }
-  }
 
   void Application::log(int verbose, const std::string& str)
   {
@@ -180,4 +131,55 @@ namespace embree
     last_virtual_memory = virtual_memory;
     last_resident_memory = resident_memory;
   }
+
+  void CommandLineParser::registerOptionAlias(const std::string& name, const std::string& alternativeName) {
+    commandLineOptionMap[alternativeName] = commandLineOptionMap[name];
+  }
+
+  void CommandLineParser::parseCommandLine(int argc, char** argv)
+  {
+    /* create stream for parsing */
+    Ref<ParseStream> stream = new ParseStream(new CommandLineStream(argc, argv));
+    
+    /* parse command line */  
+    parseCommandLine(stream, FileName());
+  }
+  
+  void CommandLineParser::parseCommandLine(Ref<ParseStream> cin, const FileName& path)
+  {
+    while (true)
+    {
+      std::string tag = cin->getString();
+      if (tag == "") return;
+      std::string tag0 = tag;
+      
+      /* remove - or -- and lookup command line option */
+      if (tag.find("-") == 0) 
+      {
+        tag = tag.substr(1);
+        if (tag.find("-") == 0) tag = tag.substr(1);
+        auto option = commandLineOptionMap.find(tag);
+      
+        /* process command line option */
+        if (option != commandLineOptionMap.end()) {
+          option->second->parse(cin,path);
+          continue;
+        }
+      }
+      
+      if (verbosity != SILENT) {
+        /* handle unknown command line options */
+        std::cerr << "unknown command line parameter: " << tag0 << " ";
+        while (cin->peek() != "" && cin->peek()[0] != '-') std::cerr << cin->getString() << " ";
+        std::cerr << std::endl;
+      }
+    }
+  }
+
+  void CommandLineParser::printCommandLineHelp()
+  {
+    for (auto& c : commandLineOptionList) {
+      std::cout << c->description << std::endl;
+    }
+  }
 }
